from unittest import TestCase

import pytest

from lotion.datetime_utils import jst_now
from lotion import Lotion
from lotion.properties import Title


@pytest.mark.api()
class TestFetchProperty(TestCase):
    DATABASE_ID = "1596567a3bbf80fc9aacdc21f9f5c516"

    def setUp(self) -> None:
        self.suite = Lotion.get_instance()
        created_page = self.suite.create_page_in_database(
            database_id=self.DATABASE_ID, properties=[Title.from_plain_text(text="テスト")]
        )
        self.page = self.suite.retrieve_page(page_id=created_page.page_id.value)
        self.now = jst_now().replace(second=0, microsecond=0)
        return super().setUp()

    def tearDown(self) -> None:
        self.suite.remove_page(self.page.page_id.value)
        return super().setUp()

    def test_作成日時と更新日時を取得する(self):
        self.assertEqual(self.page.created_at, self.now)
        self.assertEqual(self.page.updated_at, self.now)

    def test_作成ユーザと更新ユーザを取得する(self):
        expected_id = "510806db-4772-4f42-b4b6-6f81b6e8b788"
        expected_object = "user"
        self.assertEqual(self.page.created_by.id, expected_id)
        self.assertEqual(self.page.created_by.object, expected_object)
        self.assertEqual(self.page.edited_by.id, expected_id)
        self.assertEqual(self.page.edited_by.object, expected_object)

    def test_セレクトを取得する(self):
        # Given
        page_id = "15a6567a3bbf80ddb22add1abcbec3d6"
        page = self.suite.retrieve_page(page_id=page_id)

        # When
        select = page.get_select(name="セレクト")
        self.assertEqual(select.selected_name, "セレクトA")

    def test_未選択のセレクトを取得する(self):
        # Given
        page_id = "15a6567a3bbf803f9dcbdff9ef33355a"
        page = self.suite.retrieve_page(page_id=page_id)

        # When
        select = page.get_select(name="セレクト")
        self.assertIsNone(select.selected_name)



from dataclasses import dataclass

from lotion.properties.property import Property


@dataclass
class Button(Property):
    id: str
    name: str
    type: str = "button"

    @staticmethod
    def of(key: str, property:dict) -> "Button":
        return Button(id=property["id"], name=key)

    def value_for_filter(self) -> str:
        raise NotImplementedError

    def __dict__(self) -> dict:
        return {
            self.name: {
                "id": self.id,
                "type": self.type,
                "button": {},
            },
        }



from .block import Block
from .block_type import BlockType
from .video import Video
from .paragraph import Paragraph
from .quote import Quote
from .block_factory import BlockFactory
from .heading import Heading
from .divider import Divider
from .bulleted_list_item import BulletedlistItem
from .embed import Embed
from .bookmark import Bookmark
from .code import Code
from .child_database import ChildDatabase
from .to_do import ToDo
from .callout import Callout
from .child_page import ChildPage
from .column_list import ColumnList
from .image import Image
from .numbered_list_item import NumberedListItem
from .table import Table



from unittest import TestCase

import pytest
from lotion.base_page import BasePage

from lotion import Lotion


@pytest.mark.api()
class TestClientWrapper(TestCase):
    def setUp(self):
        self.suite = Lotion.get_instance()

    def test_ページを取得する(self):
        # pipenv run pytest test/test_lotion/test_lotion.py -k test_ページを取得する
        page_id = "15a6567a3bbf814b9b06e0fd3c6959e0"
        page = self.suite.retrieve_page(page_id=page_id)
        self.assertIsInstance(page, BasePage)

    def test_すべてのデータを取得できる(self):
        pass

    def test_1つの条件で絞り込む(self):
        pass
        # # Given
        # url = Url.from_url(
        #     name="Spotify", url="https://open.spotify.com/track/6tPlPsvzSM74vRVn9O5v9K"
        # )
        # filter_param = FilterBuilder().add_condition(StringCondition.equal(url)).build()

        # # 音楽のページを取得してみる
        # pages = self.suite.retrieve_database(
        #     database_id=DatabaseType.MUSIC.value,
        #     filter_param=filter_param,
        # )
        # self.assertEqual(1, len(pages))
        # self.assertEqual("タバコロード 20", pages[0].get_title().text)

    def test_タイトルを使って絞り込む_title引数(self):
        pass
        # # Given
        # title = "タバコロード 20"

        # # 音楽のページを取得してみる
        # pages = self.suite.retrieve_database(
        #     database_id=DatabaseType.MUSIC.value,
        #     title=title,
        # )
        # self.assertEqual(1, len(pages))
        # self.assertEqual("タバコロード 20", pages[0].get_title().text)

    def test_タイトルを使って絞り込む_titleをpropertyで(self):
        pass
        # Given
        # title = Title.from_plain_text(name="名前", text="タバコロード 20")
        # filter_param = (
        #     FilterBuilder().add_condition(StringCondition.equal(title)).build()
        # )

        # # 音楽のページを取得してみる
        # pages = self.suite.retrieve_database(
        #     database_id=DatabaseType.MUSIC.value,
        #     filter_param=filter_param,
        # )
        # self.assertEqual(1, len(pages))
        # self.assertEqual("タバコロード 20", pages[0].get_title().text)

    def test_返却値のモデルを指定できるようにする(self):
        pass

        # class OriginalBasePage(BasePage):
        #     pass

        # # Given
        # title = Title.from_plain_text(name="名前", text="タバコロード 20")
        # filter_param = (
        #     FilterBuilder().add_condition(StringCondition.equal(title)).build()
        # )

        # # When: モデルを指定して取得
        # pages = self.suite.retrieve_database(
        #     database_id=DatabaseType.MUSIC.value,
        #     filter_param=filter_param,
        #     page_model=OriginalBasePage,
        # )
        # self.assertIsInstance(pages[0], OriginalBasePage)

    def test_更新日時でしぼりこむ(self):
        pass
        # class OriginalBasePage(BasePage):
        #     pass

        # # Given
        # date_property = LastEditedTime.create(value=datetime(2024, 3, 17, tzinfo=JST))
        # date_property2 = LastEditedTime.create(value=datetime(2024, 3, 18, tzinfo=JST))
        # filter_param = (
        #     FilterBuilder()
        #     .add_condition(DateCondition.on_or_after(date_property))
        #     .add_condition(DateCondition.on_or_before(date_property2))
        #     .build()
        # )

        # # When: モデルを指定して取得
        # pages = self.suite.retrieve_database(
        #     database_id=DatabaseType.WEBCLIP.value,
        #     filter_param=filter_param,
        # )
        # print(pages)
        # print(len(pages))

    def test_ブロックもあわせてページをひとつ取得する(self):
        pass
        # # pytest test/notion_client_wrapper/test_client_wrapper.py::TestClientWrapper::test_ブロックもあわせてページをひとつ取得する

        # # When
        # task = self.suite.retrieve_page(
        #     page_id="21c664d6cc394d25accd77315c7a8a2e",
        #     page_model=DailyLog,
        # )
        # print(task)
        # # self.fail()

    def test_現在のタスクを取得する(self):
        pass
        # # pytest test/notion_client_wrapper/test_client_wrapper.py::TestClientWrapper::test_現在のタスクを取得する
        # from lotion.task.domain.task import ToDoTask

        # # Given
        # filter_param = {
        #     "and": [
        #         {"property": "タスク種別", "select": {"does_not_equal": "ゴミ箱"}},
        #         {"property": "実施日", "date": {"equals": "2024-03-20"}},
        #         {
        #             "or": [
        #                 {"property": "ステータス", "status": {"equals": "ToDo"}},
        #                 {"property": "ステータス", "status": {"equals": "InProgress"}},
        #             ]
        #         },
        #     ]
        # }
        # # When
        # pages = self.suite.retrieve_database(
        #     database_id=DatabaseType.TASK.value,
        #     filter_param=filter_param,
        #     page_model=ToDoTask,
        # )
        # print(pages)
        # print(len(pages))
        # # self.fail()

    @pytest.mark.skip()
    def test_select_kind_map(self):
        pass
        # """Selectの選択肢を集めるためのテスト"""
        # # pytest test/notion_client_wrapper/test_client_wrapper.py::TestClientWrapper::test_select_kind_map
        # target_database = DatabaseType.TASK_ROUTINE
        # target_select_name = "周期"

        # pages = self.suite.retrieve_database(
        #     database_id=target_database.value,
        # )

        # result = {}
        # for page in pages:
        #     select_property = page.get_select(name=target_select_name)
        #     if select_property is None:
        #         continue
        #     if select_property.selected_id in result:
        #         continue
        #     result[select_property.selected_name] = {
        #         "selected_id": select_property.selected_id,
        #         "selected_color": select_property.selected_color,
        #     }
        # # uniqueにする
        # import json

        # print(json.dumps(result, indent=2, ensure_ascii=False))

        # # 内容を確認したいので、無理やりfailさせる
        # self.fail("動作確認用。テストは失敗しても問題ありません。")

    @pytest.mark.skip()
    def test_multi_select_kind_map(self):
        pass
        """MultiSelectの選択肢を集めるためのテスト"""
        # # pytest test/notion_client_wrapper/test_client_wrapper.py::TestClientWrapper::test_multi_select_kind_map
        # target_database = DatabaseType.RECIPE
        # target_multi_select_name = "種類"

        # pages = self.suite.retrieve_database(
        #     database_id=target_database.value,
        # )

        # result = []
        # for page in pages:
        #     select_property = page.get_multi_select(name=target_multi_select_name)
        #     if select_property is None:
        #         continue
        #     values = select_property.values
        #     result.extend([{"name": value.name, "id": value.id} for value in values])
        # # uniqueにする
        # result = list({value["name"]: value for value in result}.values())
        # import json

        # print(json.dumps(result, ensure_ascii=False))

        # # 内容を確認したいので、無理やりfailさせる
        # self.fail("動作確認用。テストは失敗しても問題ありません。")

    def test_ページを作成してみる(self):
        pass
        # # pytest test/notion_client_wrapper/test_client_wrapper.py::TestClientWrapper::test_ページを作成してみる
        # title = Title.from_plain_text(name="名前", text="テストページ")
        # cover = Cover(
        #     type="external",
        #     external_url="https://i.ytimg.com/vi/82KT4FNyNdY/maxresdefault.jpg",
        # )
        # page = self.suite.create_page_in_database(
        #     database_id=DatabaseType.TASK.value,
        #     cover=cover,
        #     properties=[title],
        # )
        # print(page)

    def test_タイトルだけで検索する(self):
        pass
        # # pytest test/notion_client_wrapper/test_client_wrapper.py::TestClientWrapper::test_タイトルだけで検索する
        # page = self.suite.find_page_by_title(
        #     database_id=DatabaseType.BOOK.value,
        #     title="ジェームズ・クリアー式 複利で伸びる1つの習慣",
        #     title_key_name="Title",
        #     page_model=Book,
        # )
        # self.assertIsNotNone(page)

    def test_ページを作成する(self):
        pass
        # # pytest test/notion_client_wrapper/test_client_wrapper.py::TestClientWrapper::test_ページを作成する
        # title = Title.from_plain_text(name="名前", text="テストページ")
        # page = self.suite.create_page_in_database(
        #     database_id=DatabaseType.TASK.value,
        #     properties=[title],
        # )
        # self.assertIsNotNone(page)



from dataclasses import dataclass

from lotion.block.block import Block
from lotion.block.block_type import BlockType
from lotion.block.bookmark import Bookmark
from lotion.block.bulleted_list_item import BulletedlistItem
from lotion.block.callout import Callout
from lotion.block.child_database import ChildDatabase
from lotion.block.child_page import ChildPage
from lotion.block.code import Code
from lotion.block.column_list import ColumnList
from lotion.block.divider import Divider
from lotion.block.embed import Embed
from lotion.block.heading import Heading
from lotion.block.image import Image
from lotion.block.numbered_list_item import NumberedListItem
from lotion.block.paragraph import Paragraph
from lotion.block.quote import Quote
from lotion.block.table import Table
from lotion.block.to_do import ToDo
from lotion.block.video import Video


@dataclass
class BlockFactory:
    @staticmethod
    def create(block: dict) -> Block:
        # import json

        # print(json.dumps(block, indent=2, ensure_ascii=False))
        if block["object"] != "block":
            raise ValueError("block must be of type block")
        block_type = BlockType(block["type"])
        match block_type:
            case BlockType.VIDEO:
                return Video.of(block)
            case BlockType.PARAGRAPH:
                return Paragraph.of(block)
            case BlockType.QUOTE:
                return Quote.of(block)
            case BlockType.HEADING_1:
                return Heading.of(block)
            case BlockType.HEADING_2:
                return Heading.of(block)
            case BlockType.HEADING_3:
                return Heading.of(block)
            case BlockType.DIVIDER:
                return Divider.of(block)
            case BlockType.BULLETED_LIST_ITEM:
                return BulletedlistItem.of(block)
            case BlockType.EMBED:
                return Embed.of(block)
            case BlockType.BOOKMARK:
                return Bookmark.of(block)
            case BlockType.IMAGE:
                return Image.of(block)
            case BlockType.CODE:
                return Code.of(block)
            case BlockType.TABLE:
                return Table.of(block)
            case BlockType.NUMBERED_LIST_ITEM:
                return NumberedListItem.of(block)
            case BlockType.CHILD_DATABASE:
                return ChildDatabase.of(block)
            case BlockType.TO_DO:
                return ToDo.of(block)
            case BlockType.CALLOUT:
                return Callout.of(block)
            case BlockType.CHILD_PAGE:
                return ChildPage.of(block)
            case BlockType.COLUMN_LIST:
                return ColumnList.of(block)
            case _:
                import json

                block_json = json.dumps(block, ensure_ascii=False)
                msg = f"block type is not supported {block_type}\n{block_json}"
                raise ValueError(msg)



from datetime import date, datetime
from unittest import TestCase

import pytest

from lotion.datetime_utils import JST
from lotion import Lotion
from lotion.properties.date import Date
from lotion.properties.property import Property
from lotion.properties.title import Title


@pytest.mark.api()
class TestUpdateProperty(TestCase):
    DATABASE_ID = "1596567a3bbf80d58251f1159e5c40fa"
    PROP_NAME = "日付"

    def setUp(self) -> None:
        self.suite = Lotion.get_instance()
        created_page = self.suite.create_page_in_database(
            database_id=self.DATABASE_ID, properties=[Title.from_plain_text(text="テスト")]
        )
        self.page = self.suite.retrieve_page(page_id=created_page.page_id.value)
        return super().setUp()

    def tearDown(self) -> None:
        self.suite.remove_page(self.page.page_id.value)
        return super().setUp()

    def test_開始日を変更する(self):
        # Given
        date_ = date(2021, 1, 1)
        date_prop = Date.from_start_date(name=self.PROP_NAME, start_date=date_)

        # When, Then
        actual = self._update_page(property=date_prop)
        self.assertEqual(actual.start_date, date_)
        self.assertEqual(actual.start_time, date_)
        self.assertEqual(actual.start_datetime, datetime(2021, 1, 1, tzinfo=JST))
        self.assertEqual(actual.end_date, None)
        self.assertEqual(actual.end_datetime, None)
        self.assertEqual(actual.end_time, None)

    def test_開始時刻を変更する(self):
        # Given
        datetime_ = datetime(2021, 1, 1, 12, 34, 0, tzinfo=JST)
        date_prop = Date.from_start_date(name=self.PROP_NAME, start_date=datetime_)

        # When, Then
        actual = self._update_page(property=date_prop)
        self.assertEqual(actual.start_date, datetime_.date())
        print(actual.start_datetime)
        print(datetime_)
        self.assertEqual(actual.start_datetime, datetime_)
        self.assertEqual(actual.start_time, datetime_)
        self.assertEqual(actual.end_date, None)
        self.assertEqual(actual.end_datetime, None)
        self.assertEqual(actual.end_time, None)

    def test_開始日と終了日を変更する(self):
        # Given
        start_date = date(2021, 1, 1)
        end_date = date(2021, 1, 2)
        date_prop = Date.from_range(name=self.PROP_NAME, start=start_date, end=end_date)

        # When, Then
        actual = self._update_page(property=date_prop)
        self.assertEqual(actual.start_date, start_date)
        self.assertEqual(actual.start_time, start_date)
        self.assertEqual(actual.start_datetime, datetime(2021, 1, 1, tzinfo=JST))
        self.assertEqual(actual.end_date, end_date)
        self.assertEqual(actual.end_time, end_date)
        self.assertEqual(actual.end_datetime, datetime(2021, 1, 2, tzinfo=JST))

    def test_開始時刻と終了時刻を変更する(self):
        # Given
        start_datetime = datetime(2021, 1, 1, 12, 34, 0, tzinfo=JST)
        end_datetime = datetime(2021, 1, 1, 23, 45, 0, tzinfo=JST)
        date_prop = Date.from_range(
            name=self.PROP_NAME,
            start=start_datetime,
            end=end_datetime,
        )

        # When, Then
        actual = self._update_page(property=date_prop)
        self.assertEqual(actual.start_date, start_datetime.date())
        self.assertEqual(actual.start_time, start_datetime)
        self.assertEqual(actual.start_datetime, start_datetime)
        self.assertEqual(actual.end_date, end_datetime.date())
        self.assertEqual(actual.end_time, end_datetime)
        self.assertEqual(actual.end_datetime, end_datetime)

    def _update_page(self, property: Property):
        # When
        properties = self.page.properties.append_property(property)
        self.suite.update_page(page_id=self.page.page_id.value, properties=properties.values)

        # Then
        page = self.suite.retrieve_page(page_id=self.page.page_id.value)
        return page.get_date(name=self.PROP_NAME)



from unittest import TestCase

from lotion.filter.condition.empty_condition import EmptyCondition
from lotion.properties.relation import Relation


class TestEmptyCondition(TestCase):
    def setUp(self) -> None:
        return super().setUp()

    def test_リレーションが空である条件(self):
        # When
        condition = EmptyCondition.true(prop_name="Task List", prop_type=Relation.TYPE)

        # Then
        expected = {"property": "Task List", "relation": {"is_empty": True}}
        self.assertEqual(expected, condition.__dict__())

    def test_リレーションが空でない条件(self):
        # When
        condition = EmptyCondition.false(prop_name="Task List", prop_type=Relation.TYPE)

        # Then
        expected = {"property": "Task List", "relation": {"is_not_empty": True}}
        self.assertEqual(expected, condition.__dict__())



from dataclasses import dataclass

from lotion.properties.property import Property


@dataclass
class Rollup(Property):
    rollup_type: str
    rollup_value: str
    rollup_function: str
    type: str = "rollup"


    def __init__(self, name: str, rollup_type: str, rollup_value: str, rollup_function: str, id: str | None = None):
        self.name = name
        self.id = id
        self.rollup_type = rollup_type
        self.rollup_value = rollup_value
        self.rollup_function = rollup_function


    @staticmethod
    def of(name: str, param: dict) -> "Rollup":
        rollup_param = param["rollup"]
        if rollup_param["type"] != "number":
            raise NotImplementedError(f"Unsupported rollup type: {rollup_param['type']}")
        return Rollup(
            name=name,
            id=param["id"],
            rollup_type=rollup_param["type"],
            rollup_value=rollup_param["number"],
            rollup_function=rollup_param["function"],
        )

    def __dict__(self):
        return {
            "id": self.id,
            "type": self.type,
            "rollup": {
                "type": self.rollup_type,
                self.rollup_type: self.rollup_value,
                "function": self.rollup_function,
            },
        }

    def value_for_filter(self) -> str:
        raise NotImplementedError



from dataclasses import dataclass

from lotion.properties.property import Property


@dataclass(frozen=True)
class MultiSelectElement:
    id: str
    name: str
    color: str | None = None
    type: str = "multi_select"

    TYPE = "multi_select"

    def __dict__(self) -> dict:
        result = {
            "id": self.id,
            "name": self.name,
        }
        if self.color:
            result["color"] = self.color
        return result


@dataclass
class MultiSelect(Property):
    values: list[MultiSelectElement]
    type: str = "multi_select"

    def __init__(self, name: str, values: list[MultiSelectElement], id: str | None = None) -> None:  # noqa: A002
        self.name = name
        self.values = values
        self.id = id

    def __post_init__(self) -> None:
        if not all(isinstance(value, MultiSelectElement) for value in self.values):
            raise ValueError("All values must be MultiSelectElement instances.")

    @staticmethod
    def of(name: str, param: dict) -> "MultiSelect":
        multi_select = [
            MultiSelectElement(
                id=element["id"],
                name=element["name"],
                color=element["color"],
            )
            for element in param["multi_select"]
        ]

        return MultiSelect(
            name=name,
            values=multi_select,
            id=param["id"],
        )

    @staticmethod
    def create(name: str, values: list[dict[str, str]]) -> "MultiSelect":
        """
        Create a MultiSelect instance from a list of dictionaries.

        Args:
            name (str): Name of the property.
            values (list[dict[str, str]]): List of dictionaries. Each dictionary should have keys "id" and "name".

        Returns:
            MultiSelect: MultiSelect instance.
        """
        multi_select = [MultiSelectElement(id=element["id"], name=element["name"]) for element in values]
        return MultiSelect(
            name=name,
            values=multi_select,
        )

    def __dict__(self) -> dict:
        result = {
            "type": self.type,
            "multi_select": [e.__dict__() for e in self.values],
        }
        if self.id is not None:
            result["id"] = self.id
        return {self.name: result}

    def value_for_filter(self) -> str:
        raise NotImplementedError



from datetime import date
from unittest import TestCase

from lotion.filter.condition.date_condition import DateCondition
from lotion.filter.condition.number_condition import NumberCondition
from lotion.filter.condition.or_condition import OrCondition as Or
from lotion.filter.condition.string_condition import StringCondition
from lotion.filter import FilterBuilder
from lotion.properties.date import Date
from lotion.properties.number import Number
from lotion.properties.select import Select
from lotion.properties.status import Status
from lotion.properties.url import Url

# https://developers.notion.com/reference/post-database-query-filter


class TestFilterBuilder(TestCase):
    def setUp(self) -> None:
        return super().setUp()

    def test_シンプルな条件を作成する(self):
        # Given
        spotify_url = "https://open.spotify.com/track/6tPlPsvzSM74vRVn9O5v9K"
        url = Url.from_url(name="Spotify", url=spotify_url)

        # When
        actual = FilterBuilder().add_condition(StringCondition.equal(url)).build()
        print(actual)

        # Then
        expected = {"property": "Spotify", "url": {"equals": spotify_url}}
        self.assertEqual(expected, actual)

    def test_2つのand条件を作成する(self):
        # Given
        spotify_url = "https://open.spotify.com/track/6tPlPsvzSM74vRVn9O5v9K"
        url = Url.from_url(name="Spotify", url=spotify_url)
        number = Number.from_num(name="Number", value=1)

        # When
        actual = (
            FilterBuilder()
            .add_condition(StringCondition.equal(url))
            .add_condition(NumberCondition.equal(number))
            .build()
        )

        # Then
        expected = {
            "and": [
                {"property": "Spotify", "url": {"equals": spotify_url}},
                {"property": "Number", "number": {"equals": 1}},
            ]
        }
        self.assertEqual(expected, actual)

    def test_ステータス(self):
        # Given
        status = Status.from_status_name(name="ステータス", status_name="Done")

        # When
        actual = (
            FilterBuilder().add_condition(StringCondition.not_equal(status)).build()
        )

        # Then
        expected = {"property": "ステータス", "status": {"does_not_equal": "Done"}}

        self.assertEqual(expected, actual)

    def test_セレクト(self):
        # Given
        status = Select(name="タスク種別", selected_name="ゴミ箱", selected_id="123")

        # When
        actual = (
            FilterBuilder().add_condition(StringCondition.not_equal(status)).build()
        )

        # Then
        expected = {"property": "タスク種別", "select": {"does_not_equal": "ゴミ箱"}}

        self.assertEqual(expected, actual)

    def test_andとor条件を併用(self):
        # Given
        status = Select(name="タスク種別", selected_name="ゴミ箱", selected_id="123")
        start_date = Date.from_start_date(
            name="実施日", start_date=date.fromisoformat("2024-03-15")
        )
        status_todo = Status.from_status_name(name="ステータス", status_name="ToDo")
        status_in_progress = Status.from_status_name(
            name="ステータス", status_name="InProgress"
        )

        # When
        filter_builder = FilterBuilder()
        filter_builder = filter_builder.add_condition(StringCondition.not_equal(status))
        filter_builder = filter_builder.add_condition(DateCondition.equal(start_date))
        filter_builder = filter_builder.add_condition(
            Or.create(
                StringCondition.equal(status_todo),
                StringCondition.equal(status_in_progress),
            )
        )
        actual = filter_builder.build()

        import json

        print(json.dumps(actual, indent=2, ensure_ascii=False))

        # Then
        expected = {
            "and": [
                {"property": "タスク種別", "select": {"does_not_equal": "ゴミ箱"}},
                {"property": "実施日", "date": {"equals": "2024-03-15"}},
                {
                    "or": [
                        {"property": "ステータス", "status": {"equals": "ToDo"}},
                        {"property": "ステータス", "status": {"equals": "InProgress"}},
                    ]
                },
            ]
        }

        self.assertEqual(expected, actual)



from unittest import TestCase

import pytest

from lotion.page.page_id import PageId
from lotion import Lotion
from lotion.properties.checkbox import Checkbox
from lotion.properties.email import Email
from lotion.properties.number import Number
from lotion.properties.phone_number import PhoneNumber
from lotion.properties.property import Property
from lotion.properties.relation import Relation
from lotion.properties.status import Status
from lotion.properties.text import Text
from lotion.properties.title import Title
from lotion.properties.url import Url


@pytest.mark.api()
class TestUpdateProperty(TestCase):
    DATABASE_ID = "1596567a3bbf80d58251f1159e5c40fa"

    def setUp(self) -> None:
        self.suite = Lotion.get_instance()
        created_page = self.suite.create_page_in_database(
            database_id=self.DATABASE_ID, properties=[Title.from_plain_text(text="テスト")]
        )
        self.page = self.suite.retrieve_page(page_id=created_page.page_id.value)
        return super().setUp()

    def tearDown(self) -> None:
        self.suite.remove_page(self.page.page_id.value)
        return super().setUp()

    def test_名前を変更する(self):
        title = Title.from_plain_text(text="テスト")
        actual = self._update_page(property=title)
        self.assertEqual(actual.get_title().text, "テスト")

    def test_テキストを変更する(self):
        text_prop = Text.from_plain_text(name="テキスト", text="テスト")
        actual = self._update_page(property=text_prop)
        self.assertEqual(actual.get_text(name="テキスト").text, "テスト")

    def test_数値を変更する(self):
        number_prop = Number.from_num(name="数値", value=1)
        actual = self._update_page(property=number_prop)
        self.assertEqual(actual.get_number(name="数値").number, 1)

    def test_セレクトを変更する(self):
        self.skipTest("ユーザが名前ベースでセレクトを選べるようにしたい")

    def test_マルチセレクトを変更する(self):
        self.skipTest("ユーザが名前ベースでマルチセレクトを選べるようにしたい")

    def test_ファイルを変更する(self):
        self.skipTest("ファイルプロパティを作成するところから")

    def test_ユーザーを変更する(self):
        self.skipTest("ユーザープロパティを作成するところから")

    def test_メールを変更する(self):
        email_prop = Email.from_email(name="メール", email="sample@example.com")
        actual = self._update_page(property=email_prop)
        self.assertEqual(actual.get_email(name="メール").value, "sample@example.com")

    @pytest.mark.current()
    def test_電話番号を変更する(self):
        phone_number_prop = PhoneNumber.create(name="電話", phone_number="090-1234-5678")
        actual = self._update_page(property=phone_number_prop)
        self.assertEqual(actual.get_phone_number(name="電話").value, "090-1234-5678")

    def test_数式を変更する(self):
        self.skipTest("数式プロパティを作成するところから")

    def test_ロールアップを変更する(self):
        self.skipTest("ロールアッププロパティを作成するところから")

    def test_ステータスを変更する(self):
        status_prop = Status.from_status_name(name="ステータス", status_name="未着手")
        actual = self._update_page(property=status_prop)
        self.assertEqual(actual.get_status(name="ステータス").status_name, "未着手")

    def test_チェックボックスを変更する(self):
        checkbox_prop = Checkbox.true(name="チェックボックス")
        actual = self._update_page(property=checkbox_prop)
        self.assertEqual(actual.get_checkbox(name="チェックボックス").checked, True)

    def test_URLを変更する(self):
        url_prop = Url.from_url(name="URL", url="https://example.com")
        actual = self._update_page(property=url_prop)
        self.assertEqual(actual.get_url(name="URL").url, "https://example.com")

    @pytest.mark.minimum()
    def test_リレーションを変更する(self):
        # Given
        page_id = PageId("15a6567a3bbf814b9b06e0fd3c6959e0")
        relation_prop = Relation.from_id(name="リレーション", id=page_id.value)

        # When, Then
        actual = self._update_page(property=relation_prop)
        actual_relation = actual.get_relation(name="リレーション")
        self.assertEqual(actual_relation.id_list, [page_id.value])

    def _update_page(self, property: Property):
        # When
        properties = self.page.properties.append_property(property)
        self.suite.update_page(page_id=self.page.page_id.value, properties=properties.values)

        # Then
        return self.suite.retrieve_page(page_id=self.page.page_id.value)



from dataclasses import dataclass

from lotion.properties.property import Property


@dataclass
class Url(Property):
    url: str
    type: str = "url"

    def __init__(self, name: str, url: str, id: str | None = None):
        self.name = name
        self.url = url
        self.id = id

    @staticmethod
    def of(name: str, param: dict) -> "Url":
        return Url(
            name=name,
            url=param["url"],
            id=param["id"],
        )

    @staticmethod
    def from_url(url: str, name: str = "URL") -> "Url":
        return Url(
            name=name,
            url=url,
        )

    def value_for_filter(self) -> str:
        return self.url

    def __dict__(self):
        result = {
            "type": self.type,
            "url": self.url,
        }
        if self.id is not None:
            result["id"] = self.id
        return {
            self.name: result,
        }



from abc import ABCMeta, abstractmethod


class Property(metaclass=ABCMeta):
    id: str | None
    name: str
    type: str

    @abstractmethod
    def __dict__(self) -> dict:
        pass

    @abstractmethod
    def value_for_filter(self):  # noqa: ANN201
        pass



from dataclasses import dataclass

from lotion.page.page_id import PageId
from lotion.properties.property import Property


@dataclass
class Relation(Property):
    id_list: list[str]
    text_list: list[str]  # NOTE: Notionのデータとしては扱わない。id_listに変換するために必要になることが多いため
    type: str = "relation"
    has_more: bool = False

    TYPE = "relation"

    def __init__(  # noqa: PLR0913
        self,
        name: str,
        id: str | None = None,  # noqa: A002
        id_list: list[str] | None = None,
        text_list: list[str] | None = None,
        has_more: bool | None = None,
    ) -> None:
        self.name = name
        self.id = id
        self.id_list = id_list or []
        self.text_list = text_list or []
        self.has_more = bool(has_more)

    def is_unconverted_id_list(self) -> bool:
        """text_listがあるがid_listがない場合にTrueを返す"""
        return len(self.text_list) > 0 and len(self.id_list) == 0

    @staticmethod
    def of(name: str, property: dict[str, str]) -> "Relation":  # noqa: A002
        id_list = [r["id"] for r in property["relation"]]
        return Relation(name=name, id_list=id_list, has_more=property["has_more"])

    @staticmethod
    def from_id_list(name: str, id_list: list[str]) -> "Relation":
        return Relation(
            name=name,
            id_list=id_list,
        )

    @staticmethod
    def from_page_id_list(name: str, page_id_list: list[PageId]) -> "Relation":
        return Relation(
            name=name,
            id_list=[page_id.value for page_id in page_id_list],
        )

    @staticmethod
    def from_id(name: str, id: str) -> "Relation":
        return Relation.from_id_list(name=name, id_list=[id])

    @property
    def page_id_list(self) -> list[PageId]:
        return [PageId(id_) for id_ in self.id_list]

    def __dict__(self) -> dict:
        result = {
            "type": self.type,
            "relation": [
                {
                    "id": id,
                }
                for id in self.id_list  # noqa: A001
            ],
            "has_more": self.has_more,
        }
        if self.id is not None:
            result["relation"]["id"] = self.id
        return {
            self.name: result,
        }

    def value_for_filter(self) -> str:
        raise NotImplementedError



from dataclasses import dataclass
from enum import Enum

from lotion.filter.condition import Condition


class EmptyConditionType(Enum):
    IS_EMPTY = "is_empty"
    IS_NOT_EMPTY = "is_not_empty"


@dataclass(frozen=True)
class EmptyCondition(Condition):
    prop_name: str
    prop_type: str
    condition_type: EmptyConditionType

    @classmethod
    def true(cls: "EmptyCondition", prop_name: str, prop_type: str) -> "EmptyCondition":
        return cls(
            prop_name=prop_name,
            prop_type=prop_type,
            condition_type=EmptyConditionType.IS_EMPTY,
        )

    @classmethod
    def false(cls: "EmptyCondition", prop_name: str, prop_type: str) -> "EmptyCondition":
        return cls(
            prop_name=prop_name,
            prop_type=prop_type,
            condition_type=EmptyConditionType.IS_NOT_EMPTY,
        )

    def __dict__(self) -> dict:
        return {
            "property": self.prop_name,
            self.prop_type: {
                self.condition_type.value: True,
            },
        }



from dataclasses import dataclass
from enum import Enum

from lotion.properties import Checkbox

from lotion.filter.condition import Condition


class CheckboxConditionType(Enum):
    EQUALS = "equals"
    NOT_EQUAL = "does_not_equal"


@dataclass(frozen=True)
class CheckboxCondition(Condition):
    property_name: str
    condition_type: CheckboxConditionType
    value: bool
    # "checkbox"のみなので、property_typeは不要

    @staticmethod
    def equal(_property: Checkbox) -> "CheckboxCondition":
        return CheckboxCondition._from_property(_property, CheckboxConditionType.EQUALS)

    @staticmethod
    def not_equal(_property: Checkbox) -> "CheckboxCondition":
        return CheckboxCondition._from_property(_property, CheckboxConditionType.NOT_EQUAL)

    @staticmethod
    def _from_property(_property: Checkbox, condition_type: CheckboxConditionType) -> "CheckboxCondition":
        return CheckboxCondition(
            property_name=_property.name,
            condition_type=condition_type,
            value=_property.checked,
        )

    def __dict__(self) -> dict:
        return {
            "property": self.property_name,
            "checkbox": {
                self.condition_type.value: self.value,
            },
        }



from lotion.filter.condition import Condition, StringCondition
from lotion.properties import Property, Title


class FilterBuilder:
    def __init__(self, conditions: list[Condition] | None = None) -> None:
        self.conditions = conditions or []

    def add_condition(self, condition: Condition) -> "FilterBuilder":
        return FilterBuilder(conditions=[*self.conditions, condition])

    @staticmethod
    def build_simple_equal_condition(property: Property) -> dict:  # noqa: A002
        """指定されたプロパティが指定された値と一致する条件を作成する"""
        result = FilterBuilder().add_condition(StringCondition.equal(property=property)).build()
        if result is None:
            msg = "Filter is empty"
            raise ValueError(msg)
        return result

    @staticmethod
    def build_title_equal_condition(title: str, name: str = "名前") -> dict:
        """名前が指定された値と一致する条件を作成する"""
        title_prop = Title.from_plain_text(name=name, text=title)
        return FilterBuilder.build_simple_equal_condition(property=title_prop)

    def build(self) -> dict | None:
        if len(self.conditions) == 0:
            return None
        if len(self.conditions) == 1:
            return self.conditions[0].__dict__()
        return {
            "and": [condition.__dict__() for condition in self.conditions],
        }



import os

import requests


class UnsplashPhoto:
    BASE_URL = "https://api.unsplash.com/photos"

    def __init__(self) -> None:
        unsplash_access_key = os.getenv("UNSPLASH_ACCESS_KEY")
        if unsplash_access_key is None:
            msg = "UNSPLASH_ACCESS_KEYが設定されていません"
            raise ValueError(msg)
        self._unsplash_access_key = unsplash_access_key

    def get_random_photo_url(self, query_words: list[str]) -> str:
        """
        Get a random photo URL based on the given query words.

        Args:
            query_words (list[str]): The list of query words to search for.

        Returns:
            str: The URL of the random photo.

        Raises:
            requests.HTTPError: If the request to the Unsplash API fails.
        """
        url = self.__get_random_api_url(query_words)
        data = self.__get(url)
        return data["urls"]["full"]

    def __get(self, url: str) -> dict:
        response = requests.get(url, timeout=15)
        response.raise_for_status()
        return response.json()

    def __get_random_api_url(self, query_words: list[str]) -> str | None:
        query = ",".join(query_words)
        return f"{self.BASE_URL}/random/?client_id={self._unsplash_access_key}&query={query}"



from dataclasses import dataclass
from enum import Enum

from lotion.properties import Property
from lotion.filter.condition import Condition


class NumberConditionType(Enum):
    EQUALS= "equals"

@dataclass(frozen=True)
class NumberCondition(Condition):
    property_name: str
    property_type: str
    condition_type: NumberConditionType
    value: str

    @staticmethod
    def equal(property: Property) -> "NumberCondition":
        return NumberCondition(
            property_name=property.name,
            property_type=property.type,
            condition_type=NumberConditionType.EQUALS,
            value=property.value_for_filter(),
        )

    def __dict__(self) -> dict:
        return {
            "property": self.property_name,
            self.property_type: {
                self.condition_type.value: self.value,
            },
        }



from dataclasses import dataclass
from datetime import datetime
from enum import Enum

from lotion.properties import Property

from lotion.filter.condition import Condition


class DateConditionType(Enum):
    EQUALS = "equals"
    ON_OR_AFTER = "on_or_after"
    ON_OR_BEFORE = "on_or_before"
    BEFORE = "before"


@dataclass(frozen=True)
class DateCondition(Condition):
    property_name: str
    property_type: str
    condition_type: DateConditionType
    value: str

    @staticmethod
    def equal(property: Property) -> "DateCondition":
        return DateCondition(
            property_name=property.name,
            property_type=property.type,
            condition_type=DateConditionType.EQUALS,
            value=property.value_for_filter(),
        )

    @staticmethod
    def on_or_after(property: Property) -> "DateCondition":
        return DateCondition(
            property_name=property.name,
            property_type=property.type,
            condition_type=DateConditionType.ON_OR_AFTER,
            value=property.value_for_filter(),
        )

    @staticmethod
    def create_manually(name: str, condition_type: DateConditionType, value: datetime) -> "DateCondition":
        return DateCondition(
            property_name=name,
            property_type="date",
            condition_type=condition_type,
            value=value.isoformat(),
        )

    @staticmethod
    def before(property: Property) -> "DateCondition":
        return DateCondition(
            property_name=property.name,
            property_type=property.type,
            condition_type=DateConditionType.BEFORE,
            value=property.value_for_filter(),
        )

    @staticmethod
    def on_or_before(property: Property) -> "DateCondition":
        return DateCondition(
            property_name=property.name,
            property_type=property.type,
            condition_type=DateConditionType.ON_OR_BEFORE,
            value=property.value_for_filter(),
        )

    def __dict__(self) -> dict:
        return {
            "property": self.property_name,
            self.property_type: {
                self.condition_type.value: self.value,
            },
        }



from lotion.block import Block


class Bookmark(Block):
    caption: list
    bookmark_url: str

    def __init__(  # noqa: PLR0913
        self,
        bookmark_url: str,
        caption: list | None = None,
        id: str | None = None,  # noqa: A002
        archived: bool | None = None,
        created_time: str | None = None,
        last_edited_time: str | None = None,
        has_children: bool | None = None,
        parent: dict | None = None,
    ) -> None:
        super().__init__(id, archived, created_time, last_edited_time, has_children, parent)
        self.bookmark_url = bookmark_url
        self.caption = caption or []

    @staticmethod
    def of(block: dict) -> "Bookmark":
        bookmark = block["bookmark"]
        return Bookmark(
            id=block["id"],
            archived=block["archived"],
            created_time=block["created_time"],
            last_edited_time=block["last_edited_time"],
            has_children=block["has_children"],
            parent=block["parent"],
            caption=bookmark["caption"],
            bookmark_url=bookmark.get("url", ""),
        )

    @staticmethod
    def from_url(url: str) -> "Bookmark":
        return Bookmark(bookmark_url=url)

    @property
    def type(self) -> str:
        return "bookmark"

    def to_dict_sub(self) -> dict:
        return {
            "caption": [],
            "url": self.bookmark_url,
        }

    def to_slack_text(self) -> str:
        return self.bookmark_url



from dataclasses import dataclass
from enum import Enum

from lotion.properties import Property

from lotion.filter.condition import Condition


class StringConditionType(Enum):
    EQUALS= "equals"
    NOT_EQUAL = "does_not_equal"

@dataclass(frozen=True)
class StringCondition(Condition):
    property_name: str
    property_type: str
    condition_type: StringConditionType
    value: str

    @staticmethod
    def equal(property: Property) -> "StringCondition":
        return StringCondition._from_property(property, StringConditionType.EQUALS)

    @staticmethod
    def not_equal(property: Property) -> "StringCondition":
        return StringCondition._from_property(property, StringConditionType.NOT_EQUAL)

    @staticmethod
    def _from_property(property: Property, condition_type: StringConditionType) -> "StringCondition":
        return StringCondition(
            property_name=property.name,
            property_type=property.type,
            condition_type=condition_type,
            value=property.value_for_filter(),
        )

    def __dict__(self) -> dict:
        return {
            "property": self.property_name,
            self.property_type: {
                self.condition_type.value: self.value,
            },
        }



from dataclasses import dataclass

from lotion.filter.condition import Condition


@dataclass(frozen=True)
class OrCondition(Condition):
    conditions: list[Condition]

    @staticmethod
    def create(*conditions: Condition) -> "OrCondition":
        return OrCondition(conditions=list(conditions))


    def __dict__(self) -> dict:
        return {
            "or": [c.__dict__() for c in self.conditions],
        }



from dataclasses import dataclass
from enum import Enum

from lotion.filter.condition.or_condition import OrCondition
from lotion.properties.relation import Relation

from lotion.filter.condition import Condition


class RelationConditionType(Enum):
    CONTAINS = "contains"


@dataclass(frozen=True)
class RelationCondition(Condition):
    property_name: str
    property_type: str
    value: str
    condition_type: RelationConditionType

    @classmethod
    def contains(cls: "RelationCondition", relation: Relation) -> OrCondition:
        id_list = relation.id_list
        relation_condition_list = [
            RelationCondition(
                property_name=relation.name,
                property_type=relation.type,
                value=id_,
                condition_type=RelationConditionType.CONTAINS,
            )
            for id_ in id_list
        ]
        return OrCondition(relation_condition_list)

    def __dict__(self) -> dict:
        return {
            "property": self.property_name,
            self.property_type: {
                self.condition_type.value: self.value,
            },
        }



def pytest_configure(config):
    config.addinivalue_line("markers", "current: As current ones")
    config.addinivalue_line("markers", "learning: As learning exercises")
    config.addinivalue_line("markers", "api: As using the Notion API")
    config.addinivalue_line("markers", "slow: As slow ones")
    config.addinivalue_line("markers", "minimum: 最低限やっておきたいテスト")



from unittest import TestCase

from lotion.filter.condition.relation_condition import RelationCondition
from lotion.properties.relation import Relation


class TestRelationCondition(TestCase):
    def setUp(self) -> None:
        return super().setUp()

    def test_contains(self):
        # When
        relation = Relation.from_id_list(name="プロジェクト", id_list=["5673db2d520f48fbad6622a38cf2ecad"])
        condition = RelationCondition.contains(relation)

        # Then
        expected = {"or": [{"property": "プロジェクト", "relation": {"contains": "5673db2d520f48fbad6622a38cf2ecad"}}]}
        self.assertEqual(expected, condition.__dict__())



from unittest import TestCase

import pytest
from lotion.block import Block, Bookmark, BulletedlistItem, Callout, Code, Divider, Embed, Heading, Image, NumberedListItem, Paragraph, Quote, ToDo, Video
from lotion import Lotion


@pytest.mark.api()
class TestAppendBlock(TestCase):
    PAGE_ID = "1596567a3bbf8049803de1ffe3616d9e"

    def setUp(self):
        self.suite = Lotion.get_instance()
        self.suite.clear_page(self.PAGE_ID)

    def test_パラグラフを追加する(self):
        text_block = Paragraph.from_plain_text(text="テスト")
        self._append_block_test(block=text_block)

    def test_ブックマークを追加する(self):
        bookmark = Bookmark.from_url(url="https://www.google.com/")
        self._append_block_test(block=bookmark)

    def test_リストを追加する(self):
        list_block = BulletedlistItem.from_plain_text(text="テスト1")
        self._append_block_test(block=list_block)
        list_block = BulletedlistItem.from_plain_text(text="テスト2")
        self._append_block_test(block=list_block)

    def test_区切り線を追加する(self):
        divider = Divider()
        self._append_block_test(block=divider)

    def test_埋め込みを追加する(self):
        embed = Embed.from_url(url="https://www.google.com/")
        self._append_block_test(block=embed)

    def test_見出しを追加する(self):
        header = Heading.from_plain_text(heading_size=1, text="テスト")
        self._append_block_test(block=header)

    def test_画像を追加する(self):
        url = "https://d3swar8tu7yuby.cloudfront.net/IMG_6286_thumb.jpg"
        image = Image.from_external_url(url=url)
        self._append_block_test(block=image)

    def test_番号付きリストを追加する(self):
        numberd_list_block = NumberedListItem.from_plain_text(text="テスト1")
        self._append_block_test(block=numberd_list_block)

        numberd_list_block = NumberedListItem.from_plain_text(text="テスト2")
        self._append_block_test(block=numberd_list_block)

    def test_引用を追加する(self):
        quote = Quote.from_plain_text(text="テスト")
        self._append_block_test(block=quote)

    def test_TODOリストを追加する(self):
        todo = ToDo.from_plain_text(text="テスト1")
        self._append_block_test(block=todo)
        todo = ToDo.from_plain_text(text="テスト2", checked=True)
        self._append_block_test(block=todo)

    def test_動画を追加する(self):
        url = "https://www.youtube.com/watch?v=L5mF2uBKhS8"
        video = Video.from_external_url(url=url)
        self._append_block_test(block=video)

    def test_コールアウトを追加する(self):
        callout = Callout.from_plain_text(text="テスト")
        self._append_block_test(block=callout)

    def test_コードを追加する(self):
        code = Code.from_plain_text(text="print('hello')", language="python")
        self._append_block_test(block=code)

    def _append_block_test(self, block):
        # When, Then
        actual = self.suite.append_block(block_id=self.PAGE_ID, block=block)
        self.assertTrue("results" in actual)
        block_by_get = self._get_first_block()
        self.assertIsInstance(block_by_get, block.__class__)

    def _get_first_block(self) -> Block:
        page = self.suite.retrieve_page(self.PAGE_ID)
        return page.block_children[0]



from unittest import TestCase

from lotion.block import BulletedlistItem
from lotion.block.rich_text import RichText


class TestBulletedlistItem(TestCase):
    def test(self) -> None:
        # Given
        rich_text_foo = RichText.from_plain_text("foo")
        suite = BulletedlistItem.from_rich_text(rich_text_foo)

        # When
        # Then
        self.assertEqual("- foo", suite.to_slack_text())



from dataclasses import dataclass

from lotion.block.rich_text.rich_text_element import RichTextElement, RichTextTextElement


@dataclass(frozen=True)
class RichText:
    elements: list[RichTextElement]

    @staticmethod
    def from_entity(rich_text: list) -> "RichText":
        return RichText(elements=list(map(lambda x: RichTextElement.from_entity(x), rich_text)))

    @staticmethod
    def from_plain_text(text: str) -> "RichText":
        rich_text_element = RichTextTextElement.of(content=text)
        return RichText(elements=[rich_text_element])

    @classmethod
    def from_plain_link(cls, text: str, url: str) -> "RichText":
        rich_text_element = RichTextTextElement.of(content=text, link_url=url)
        return cls(elements=[rich_text_element])

    def to_plain_text(self) -> str:
        return "".join(map(lambda x: x.to_plain_text(), self.elements))

    def to_dict(self) -> list[dict]:
        return list(map(lambda x: x.to_dict(), self.elements))

    def to_slack_text(self) -> str:
        return "".join([x.to_slack_text() for x in self.elements])



from .condition import Condition
from .checkbox_condition import CheckboxCondition, CheckboxConditionType
from .date_condition import DateCondition, DateConditionType
from .number_condition import NumberCondition, NumberConditionType
from .string_condition import StringCondition, StringConditionType
from .empty_condition import EmptyCondition, EmptyConditionType
from .relation_condition import RelationCondition, RelationConditionType
from .or_condition import OrCondition as Or



from unittest import TestCase

from lotion.block.to_do import ToDo


class TestToDo(TestCase):
    def setUp(self) -> None:
        return super().setUp()

    def test(self):
        # Given
        block_dict = {
            "object": "block",
            "id": "56696b99-a01d-4acc-8e8a-438e4aece59f",
            "parent": {
                "type": "page_id",
                "page_id": "470e7491-f533-4836-8f27-213ed8947ba6",
            },
            "created_time": "2024-06-04T01:38:00.000Z",
            "last_edited_time": "2024-06-04T01:39:00.000Z",
            "created_by": {
                "object": "user",
                "id": "6f877603-07b9-4467-b811-31706eeb640c",
            },
            "last_edited_by": {
                "object": "user",
                "id": "6f877603-07b9-4467-b811-31706eeb640c",
            },
            "has_children": False,
            "archived": False,
            "in_trash": False,
            "type": "to_do",
            "to_do": {
                "rich_text": [
                    {
                        "type": "text",
                        "text": {"content": "test", "link": None},
                        "annotations": {
                            "bold": False,
                            "italic": False,
                            "strikethrough": False,
                            "underline": False,
                            "code": False,
                            "color": "default",
                        },
                        "plain_text": "test",
                        "href": None,
                    }
                ],
                "checked": False,
                "color": "default",
            },
        }

        # When
        suite = ToDo.of(block_dict)
        actual = suite.to_dict()

        # Then
        self.assertTrue(actual["type"] == "to_do")
